import asyncio
import inspect
import re
import sys
import typing
from contextlib import suppress
from dataclasses import dataclass, field
from functools import partial, wraps
from types import FunctionType, MethodType, ModuleType
from typing import (
    Any,
    Callable,
    Dict,
    Generic,
    List,
    Mapping,
    Optional,
    Type,
    TypeVar,
    Union,
    cast,
    TypedDict,
    overload
)


from arclet.alconna.args import ArgFlag, Args, TAValue, Arg
from arclet.alconna.action import Action
from arclet.alconna.arparma import Arparma, ArparmaBehavior
from arclet.alconna.base import Option, Subcommand
from arclet.alconna.model import OptionResult
from arclet.alconna.core import Alconna
from arclet.alconna.exceptions import NullMessage
from arclet.alconna.manager import command_manager, ShortcutArgs
from arclet.alconna.typing import TDC, KeyWordVar, MultiVar, CommandMeta, AllParam
from nepattern import ANY, all_patterns, type_parser, RawStr
from tarina import split, split_once, init_spec, lang, Empty
from typing_extensions import get_origin, NotRequired

T = TypeVar("T")
TCallable = TypeVar("TCallable", bound=Callable)

PARSER_TYPE = Callable[
    [Callable[..., T], Arparma, Dict[str, Any], asyncio.AbstractEventLoop],
    T,
]


def default_parser(
    func: Callable[..., T],
    result: Arparma,
    local_arg: Dict[str, Any],
    loop: asyncio.AbstractEventLoop,
) -> T:
    return result.call(func)


class Executor(Generic[T]):
    """
    以 click-like 方法创建的 Alconna 结构体, 可以被视为一类 CommanderHandler
    """

    command: Alconna
    parser_func: Callable[
        [Callable[..., T], Arparma, Dict[str, Any], asyncio.AbstractEventLoop],
        T,
    ]
    local_args: Dict[str, Any]
    exec_target: Callable[..., T]

    def __init__(self, command: Alconna, target: Callable):
        self.command = command
        self.exec_target = target
        self.parser_func = default_parser
        self.local_args = {}

    def set_local_args(self, local_args: Optional[Dict[str, Any]] = None):
        """
        设置本地参数

        Args:
            local_args (Optional[Dict[str, Any]]): 本地参数
        """
        self.local_args = local_args or {}

    def set_parser(self, parser_func: PARSER_TYPE):
        """
        设置解析器

        Args:
            parser_func (PARSER_TYPE): 解析器, 接受的参数必须为 (func, args, local_args, loop)
        """
        self.parser_func = parser_func
        return self

    def __call__(self, message: TDC) -> Arparma[TDC]:
        if not self.exec_target:
            raise RuntimeError(lang.require("tools", "construct.decorate_error"))
        result = self.command.parse(message)
        if result.matched:
            self.parser_func(
                self.exec_target,
                result,
                self.local_args,
                asyncio.get_event_loop(),
            )
        return result

    def from_commandline(self):
        """从命令行解析参数"""
        if not self.command:
            raise RuntimeError(lang.require("tools", "construct.decorate_error"))
        args = sys.argv[1:]
        args.insert(0, self.command.command)
        return self.__call__(" ".join(args))


# ----------------------------------------
# click-like
# ----------------------------------------


class AlconnaDecorate:
    """
    Alconna Click-like 构造方法的生成器

    Examples:
        >>> cli = AlconnaDecorate()
        >>> @cli.command()
        ... @cli.option("--name|-n", Args["name", str, "your name"])
        ... @cli.option("--age|-a", Args["age", int, "your age"])
        ... def hello(name: str, age: int):
        ...     print(f"Hello {name}, you are {age} years old.")
        ...
        >>> hello("hello --name Alice --age 18")

    Attributes:
        namespace (str): 命令的命名空间
    """

    namespace: str
    building: bool
    buffer: Dict[str, Any]
    default_parser: PARSER_TYPE

    def __init__(self, namespace: str = "Alconna"):
        """
        初始化构造器

        Args:
            namespace (str): 命令的命名空间
        """
        self.namespace = namespace
        self.building = False
        self.buffer = {}
        self.default_parser = default_parser

    def command(
        self,
        name: Optional[str] = None,
        headers: Optional[List[Any]] = None,
    ) -> Callable[[Callable[..., T]], Executor[T]]:
        """
        开始构建命令

        Args:
            name (Optional[str]): 命令名称
            headers (Optional[List[Any]]): 命令前缀
        """
        self.building = True

        def wrapper(func: Callable[..., T]) -> Executor[T]:
            alc = Alconna(
                name or func.__name__,
                headers or [],
                self.buffer.pop("args", Args()),
                *self.buffer.pop("options", []),
                meta=self.buffer.pop("meta", CommandMeta()),
                namespace=self.namespace,
            )
            if alc.meta.example and "$" in alc.meta.example:
                alc.meta.example = alc.meta.example.replace("$", str(alc.prefixes[0]) if alc.prefixes else "")
            self.building = False
            return Executor(alc, func).set_parser(self.default_parser)

        return wrapper

    @init_spec(Option, True)
    def option(self, opt: Option) -> Callable[[TCallable], TCallable]:
        """
        添加命令选项

        """
        if not self.building:
            raise RuntimeError(lang.require("tools", "construct.decorate_error"))

        def wrapper(func: TCallable) -> TCallable:
            self.buffer.setdefault("options", []).append(opt)
            return func

        return wrapper

    @init_spec(Subcommand, True)
    def subcommand(self, sub: Subcommand) -> Callable[[TCallable], TCallable]:
        """
        添加命令选项

        """
        if not self.building:
            raise RuntimeError(lang.require("tools", "construct.decorate_error"))

        def wrapper(func: TCallable) -> TCallable:
            self.buffer.setdefault("options", []).append(sub)
            return func

        return wrapper

    def main_args(self, args: Args) -> Callable[[TCallable], TCallable]:
        """
        添加命令参数

        Args:
            args (Args): 参数
        """
        if not self.building:
            raise RuntimeError(lang.require("tools", "construct.decorate_error"))

        def wrapper(func: TCallable) -> TCallable:
            self.buffer["args"] = args
            return func

        return wrapper

    @init_spec(Args, True)
    def argument(self, arg: Args) -> Callable[[TCallable], TCallable]:
        """
        添加命令参数
        """
        if not self.building:
            raise RuntimeError(lang.require("tools", "construct.decorate_error"))

        def wrapper(func: TCallable) -> TCallable:
            if args := self.buffer.get("args"):  # type: ignore
                args: Args
                args.__merge__(arg)
            else:
                self.buffer["args"] = Args().__merge__(arg)
            return func

        return wrapper

    def meta(self, content: CommandMeta) -> Callable[[TCallable], TCallable]:
        """
        添加命令元数据
        """
        if not self.building:
            raise RuntimeError(lang.require("tools", "construct.decorate_error"))

        def wrapper(func: TCallable) -> TCallable:
            self.buffer["meta"] = content
            return func

        return wrapper

    def help(
        self,
        description: str,
        usage: Optional[str] = None,
        example: Optional[str] = None,
    ) -> Callable[[TCallable], TCallable]:
        """
        添加命令元数据
        """
        if not self.building:
            raise RuntimeError(lang.require("tools", "construct.decorate_error"))

        def wrapper(func: TCallable) -> TCallable:
            self.buffer["meta"] = CommandMeta(description, usage, example)
            return func

        return wrapper

    def set_default_parser(self, parser_func: PARSER_TYPE):
        """
        设置默认的参数解析器

        Args:
            parser_func (PARSER_TYPE): 参数解析器, 接受的参数必须为 (func, args, local_args, loop)
        """
        self.default_parser = parser_func
        return self


def args_from_list(args: List[List[str]], custom_types: Dict[str, type]) -> Args:
    """
    从处理好的字符串列表中生成Args

    Examples:
        >>> args_from_list([["foo", "str"], ["bar", "digit", "123"]], {"digit":int})
        Args(foo: str, bar: int = 123)
        >>> args_from_list([["foo", "str+"]])
        Args(foo: (str+))
    """
    _args = Args()
    for arg in args:
        if (_le := len(arg)) == 0:
            raise NullMessage
        default = arg[2].strip(" ") if _le > 2 else Empty
        name = arg[0].strip(" ")
        value = (
            AllParam
            if name.startswith("...") else
            (arg[1].strip(" ") if _le > 1 else ANY)
        )
        name = name.replace("...", "")
        _multi, _kw, _slice = "", False, -1
        if isinstance(value, str):
            if mat := re.match(
                r"^(?P<name>.+?)(?P<multi>[+*]+)(\[)?(?P<slice>\d*)(])?$", value
            ):
                value = mat["name"]
                _multi = mat["multi"][0]
                _kw = len(mat["multi"]) > 1
                _slice = int(mat["slice"] or -1)
            with suppress(NameError, ValueError, TypeError):
                _types = custom_types.copy()
                _types.update(typing.__dict__)
                value = all_patterns().get(value, None) or type_parser(eval(value, custom_types))  # type: ignore
                default = (
                    (get_origin(value.origin) or value.origin)(default)
                    if default is not Empty
                    else default
                )
            if _multi:
                value = MultiVar(
                    KeyWordVar(value) if _kw else value,
                    _slice if _slice > 1 else _multi,  # type: ignore
                )
        _args.add(name, value=value, default=default)  # type: ignore
    return _args


def args_from_string(string: str, formats: Mapping[str, Union[TAValue, Args, Arg]], args: Args):
    if mat := re.match(r"^\{(?P<pattern>.+?)}$", string):
        pat = mat["pattern"]
        if pat in formats:
            value = formats[pat]
            if isinstance(value, (Args, Arg)):
                args.__merge__(value)
            else:
                args.__merge__([pat, value])
        else:
            part = re.split("[:=]", pat)
            if len(part) == 1:
                args.__merge__([part[0], ANY])
            else:
                args.__merge__(args_from_list([part], {}))
    else:
        args.__merge__([string, RawStr(string)])


def alconna_from_format(
    format_string: str,
    format_args: Optional[Mapping[str, Union[TAValue, Args, Arg]]] = None,
    meta: Optional[CommandMeta] = None,
    union: bool = True,
) -> "Alconna":
    """
    以格式化字符串的方式构造 Alconna

    该方法建议使用多个重名的命令时使用

    Examples:

        >>> alc1 = AlconnaFormat(
        ...     "lp user {target:str} perm set {perm:str} {default}",
        ...     {"default": Args["val", bool, True]},
        ... )
        >>> alc2 = AlconnaFormat(
        ...     "lp user {target:str} perm del {perm:str}",
        ... )
        >>> alc3 = AlconnaFormat(
        ...     "lp user {target:str} perm info {perm:str}"
        ... )
        >>> alc1.parse("lp user AAA perm set admin.all False")
        >>> alc1.parse("lp user AAA perm info admin.all")
    """
    formats = format_args or {}
    _key_ref = 0
    strings = split(format_string.replace("{", "\"{").replace("}", "}\""), (" ",))
    command = strings.pop(0)
    data = []
    if mat := re.match(r"^\[(.+?)]$", command):
        data.append([i.strip() for i in mat["data"].split("|")])
    else:
        data.append(command)
    main_args = Args()
    _finish_arg = False
    _stack = []
    for string in strings:
        if string.startswith("-"):
            _finish_arg = True
            _stack.append(string)
            continue
        if not _finish_arg:
            args_from_string(string, formats, main_args)
        elif not _stack:
            raise ValueError(lang.require("tools", "construct.format_error".format(target=string)))
        else:
            singles = _stack[:-1]
            name = _stack[-1]
            _opt_args = Args()
            args_from_string(string, formats, _opt_args)
            data.extend(Option(single) for single in singles)
            data.append(Option(name, _opt_args))
    alc = Alconna(main_args, *data, meta=meta)
    if union:
        with suppress(ValueError):
            cmd = command_manager.get_command(alc.path)
            cmd = cmd | alc
            return cmd
    return alc


class AlconnaString:
    """以纯字符串的形式构造Alconna的简易方式, 或者说是koishi-like的方式

    Examples:

        >>> alc = (
        ...     AlconnaString("test <message:str:hello> #HELP_STRING")
        ...     .option("foo", "-f <val:bool>")
        ...     .option("bar", "-bar <bar:str> [baz:int]")
        ...     .option("qux", default=123)
        ...     .build()
        ... )
        >>> alc.parse("test abcd --foo True")
    """
    @staticmethod
    def args_gen(pattern: str, types: dict):
        args = Args()
        temp = []
        quote = False
        for char in pattern:
            if char == " " and not quote:
                if temp:
                    args.__merge__([temp[0], RawStr(temp[0])])
                    temp.clear()
                continue
            if char in {"<", "["}:  # start
                quote = True
                temp.append("")
            elif char == ">":
                args.__merge__(args_from_list([temp], types.copy()))
                temp.clear()
                quote = False
            elif char == "]":
                temp[0] = f"{temp[0]};?"
                args.__merge__(args_from_list([temp], types.copy()))
                temp.clear()
                quote = False
            elif char in {":", "="}:
                temp.append("")
            elif not temp:
                temp.append(char)
            else:
                temp[-1] += char
        return args

    def __init__(self, command: str, help_text: Optional[str] = None):
        """创建 AlconnaString

        Args:
            command (str): 命令字符串, 例如 `test <message:str:hello> #HELP_STRING`
            help_text (Optional[str], optional): 选填的命令的帮助文本.
        """
        self.buffer = {}
        self.options = []
        self.shortcuts = {}
        self.actions = []
        head, others = split_once(command, (" ",))
        if mat := re.match(r"^\[(.+?)]$", head):
            self.buffer["prefixes"] = mat[1].split("|")
        else:
            self.buffer["command"] = head.lstrip()
        self.meta = CommandMeta(description=help_text or self.buffer.get("command"), fuzzy_match=True)
        if help_string := re.findall(r"(?: )#(.+)$", others):  # noqa
            self.meta.description = help_string[0]
            others = others[: -len(help_string[0]) - 1].rstrip()
        custom_types = getattr(inspect.getmodule(inspect.stack()[1][0]), "__dict__", {})
        self.buffer["main_args"] = self.args_gen(others, custom_types.copy())

    def alias(self, alias: str):
        self.shortcut(alias, {"prefix": True})
        return self

    def option(self, name: str, opt: Optional[str] = None, default: Any = None, action: Optional[Action] = None):
        """添加一个选项

        Args:
            name (str): 选项的实际名称
            opt (Optional[str], optional): 选项的字符串, 例如 `--foo -f <val:bool>`.
            default (Any, optional): 选项的默认值.
            action (Optional[Action], optional): 选项的动作.
        """
        _default = default
        if isinstance(default, dict):
            _default = OptionResult(args=default)
        if opt is None:
            self.options.append(
                Option(name, default=_default, action=action)
            )
            return self
        help_text = None
        if help_string := re.findall(r"(?: )#(.+)$", opt):  # noqa
            help_text = help_string[0]
            opt = opt[: -len(help_string[0]) - 1].rstrip()
        parts = split(opt, (" ",))
        aliases = [f"--{name}"]
        index = 0
        for part in parts:
            if part.startswith("<") or part.startswith("["):
                break
            aliases.append(part)
            index += 1
        _args = Args()
        if parts[index:]:
            custom_types = getattr(inspect.getmodule(inspect.stack()[1][0]), "__dict__", {})
            _args = self.args_gen(" ".join(parts[index:]), custom_types.copy())
        _opt = Option("|".join(aliases), _args, dest=name, default=_default, action=action, help_text=help_text)
        self.options.append(_opt)
        return self

    def subcommand(self, name: str, default: Any = None):
        """添加一个子命令

        Args:
            name (str): 子命令的名称
            default (Any, optional): 子命令的默认值.
        """
        _default = default
        if isinstance(default, dict):
            _default = OptionResult(args=default)
        help_text = None
        if help_string := re.findall(r"(?: )#(.+)$", name):  # noqa
            help_text = help_string[0]
            name = name[: -len(help_string[0]) - 1].rstrip()
        parts = split(name, (" ",))
        aliases = []
        index = 0
        for part in parts:
            if part.startswith("<") or part.startswith("["):
                break
            aliases.append(part)
            index += 1
        _args = Args()
        if parts[index:]:
            custom_types = getattr(inspect.getmodule(inspect.stack()[1][0]), "__dict__", {})
            _args = self.args_gen(" ".join(parts[index:]), custom_types.copy())
        _opt = Subcommand("|".join(aliases), _args, dest=name, default=_default, help_text=help_text)
        return self

    def usage(self, content: str):
        """设置命令的使用方法"""
        self.meta.usage = content
        return self

    def example(self, content: str):
        """设置命令的使用示例"""
        self.meta.example = content
        return self

    def shortcut(self, key: str, args: Optional[ShortcutArgs] = None):
        """设置命令的快捷方式"""
        self.shortcuts[key] = args or {}
        return self

    def action(self, func: Callable):
        """设置命令的动作"""
        self.actions.append(func)
        return self

    def build(self):
        """构造为 Alconna 对象"""
        alc = Alconna(*self.buffer.values(), *self.options, meta=self.meta)
        for key, args in self.shortcuts.items():
            alc.shortcut(key, args)
        for action in self.actions:
            alc.bind()(action)
        return alc


class MountConfig(TypedDict):
    prefixes: NotRequired[List[str]]
    raise_exception: NotRequired[bool]
    description: NotRequired[str]
    namespace: NotRequired[str]
    command: NotRequired[str]


config_keys = ("prefixes", "raise_exception", "description", "namespace", "command")


def visit_config(obj: Any, base: Optional[MountConfig] = None) -> MountConfig:
    result: MountConfig = base or {}
    if isinstance(obj, (FunctionType, MethodType)):
        codes, _ = inspect.getsourcelines(obj)
        _get_config = False
        _start_indent = 0
        for line in codes:
            indent = len(line) - len(line.lstrip())
            if line.lstrip().startswith("class") and line.lstrip().rstrip(
                "\n"
            ).endswith("Config:"):
                _get_config = True
                _start_indent = indent
                continue
            if _get_config:
                if indent == _start_indent:
                    break
                if line.lstrip().startswith("def"):
                    continue
                _contents = re.split(r"\s*=\s*", line.strip())
                if len(_contents) == 2 and _contents[0] in config_keys:
                    result[_contents[0]] = eval(_contents[1])  # type: ignore
    elif kss := inspect.getmembers(
        obj, lambda x: inspect.isclass(x) and x.__name__.endswith("Config")
    ):
        ks = kss[0][1]
        result.update({k: getattr(ks, k) for k in config_keys if k in dir(ks)})  # type: ignore
    return result


@dataclass(unsafe_hash=True)
class CallbackHandler(ArparmaBehavior):
    main_call: Optional[Callable] = field(default=None)
    options: Dict[str, Callable] = field(default_factory=dict, hash=False)
    results: Dict[str, Any] = field(default_factory=dict, hash=False)

    def before_operate(self, interface: Arparma):
        super().before_operate(interface)
        self.results.clear()

    def operate(self, interface: Arparma):
        if call := self.main_call:
            call(**interface.main_args)
        for path, action in self.options.items():
            if (d := interface.query(path, None)) is not None:
                self.results[action.__qualname__] = action(**d)


class SubClassMounter(Subcommand):
    instance: Any

    def _get_instance(self):
        return self.instance

    def _inject_instance(self, target: Callable):
        @wraps(target)
        def wrapper(*args, **kwargs):
            return target(self._get_instance(), *args, **kwargs)

        return wrapper

    def __init__(self, mount_cls: Type, upper_handler: CallbackHandler, upper_path: str):
        self.mount_cls = mount_cls
        config = visit_config(mount_cls)
        members = inspect.getmembers(
            mount_cls, lambda x: inspect.isfunction(x) or inspect.ismethod(x)
        )

        _options = []
        main_help_text = (
            mount_cls.__doc__ or mount_cls.__init__.__doc__ or mount_cls.__name__
        )

        main_args = Args.from_callable(mount_cls.__init__)[0]

        def _main_func(**kwargs):
            if hasattr(self, "instance"):
                for k, v in kwargs.items():
                    setattr(self.instance, k, v)
            else:
                self.instance = mount_cls(**kwargs)
                for key, value in kwargs.items():
                    self.args[key].field.default = value  # type: ignore

        path = f"subcommands.{upper_path}.{mount_cls.__name__}" if upper_path else f"subcommands.{mount_cls.__name__}"
        upper_handler.options[f"{path}.args"] = _main_func
        for name, func in filter(lambda x: not x[0].startswith("_"), members):
            help_text = func.__doc__ or name
            _opt_args, method = Args.from_callable(func)
            if method:
                func = self._inject_instance(func)
            _options.append(
                Option(name, _opt_args, help_text=help_text)
            )
            upper_handler.options[f"{path}.options.{name}.args"] = func

        _options.extend(
            SubClassMounter(cls, upper_handler, path)
            for name, cls in inspect.getmembers(mount_cls, inspect.isclass)
            if not name.startswith("_") and not name.endswith("Config")
        )

        super().__init__(
            config.get("command", mount_cls.__name__),
            main_args,
            *_options,
            help_text=config.get("description", main_help_text),
        )


class FuncMounter(Alconna[TDC], Generic[T, TDC]):
    def __init__(
        self, func: Callable[..., T], config: Optional[MountConfig] = None
    ):
        config = visit_config(func, config)
        func_name = func.__name__
        if func_name.startswith("_"):
            raise ValueError(lang.require("tools", "construct.func_name_error"))
        _args, method = Args.from_callable(func)
        if method and isinstance(func, MethodType):
            self.instance = func.__self__
            func = cast(FunctionType, partial(func, self.instance))
        super(FuncMounter, self).__init__(
            config.get("prefixes", []),
            config.get("command", func_name),
            _args,
            meta=CommandMeta(
                description=config.get("description", func.__doc__ or func_name),
                raise_exception=config.get("raise_exception", True),
            ),
            namespace=config.get("namespace", None),
        )
        self.bind()(func)

    @property
    def exec_result(self) -> Dict[str, T]:
        return {ext.target.__name__: res for ext, res in self._executors.items() if res is not None}


class ModuleMounter(Alconna):
    def __init__(self, module: ModuleType, config: Optional[MountConfig] = None):
        self.mount_cls = module.__class__
        self.instance = module
        config = config or visit_config(module, config)
        _options = []
        members = inspect.getmembers(
            module, lambda x: inspect.isfunction(x) or inspect.ismethod(x)
        )
        self.cb_behavior = CallbackHandler()
        for name, func in members:
            if name.startswith("_") or func.__name__.startswith("_"):
                continue
            help_text = func.__doc__ or name
            _opt_args, method = Args.from_callable(func)
            if method:
                func = partial(func, func.__self__)
            _options.append(
                Option(
                    name, args=_opt_args, help_text=help_text
                )
            )
            self.cb_behavior.options[f"options.{name}.args"] = func
        _options.extend(
            SubClassMounter(cls, self.cb_behavior, "")
            for name, cls in inspect.getmembers(module, inspect.isclass)
            if not name.startswith("_") and not name.endswith("Config")
        )
        super().__init__(
            config.get("command", module.__name__),
            config.get("prefixes", []),
            *_options,
            namespace=config.get("namespace", None),
            meta=CommandMeta(
                description=config.get(
                    "description", module.__doc__ or module.__name__
                ),
                raise_exception=config.get("raise_exception", True),
            ),
            behaviors=[self.cb_behavior],
        )

    def get_result(self, func: Callable):
        return self.cb_behavior.results.get(func.__qualname__)


class ClassMounter(Alconna[TDC], Generic[T, TDC]):
    mount_cls: Type[T]
    instance: T

    def _get_instance(self) -> T:
        return self.instance

    def _inject_instance(self, target: Callable):
        @wraps(target)
        def wrapper(*args, **kwargs):
            return target(self._get_instance(), *args, **kwargs)

        return wrapper

    def __init__(self, mount_cls: Type[T], config: Optional[MountConfig] = None):
        self.mount_cls = mount_cls
        config = config or visit_config(mount_cls, config)
        members = inspect.getmembers(
            mount_cls, lambda x: inspect.isfunction(x) or inspect.ismethod(x)
        )

        _options = []
        main_help_text = (
            mount_cls.__doc__ or mount_cls.__init__.__doc__ or mount_cls.__name__
        )

        main_args = Args.from_callable(mount_cls.__init__)[0]

        def _main_func(**kwargs):
            if hasattr(self, "instance"):
                for k, v in kwargs.items():
                    setattr(self.instance, k, v)
            else:
                self.instance = mount_cls(**kwargs)
                for key, value in kwargs.items():
                    self.args[key].field.default = value  # type: ignore

        self.cb_behavior = CallbackHandler(main_call=_main_func)
        for name, func in filter(lambda x: not x[0].startswith("_"), members):
            help_text = func.__doc__ or name
            _opt_args, method = Args.from_callable(func)
            if method:
                func = self._inject_instance(func)
            _options.append(
                Option(name, _opt_args, help_text=help_text)
            )
            self.cb_behavior.options[f"options.{name}.args"] = func

        _options.extend(
            SubClassMounter(cls, self.cb_behavior, "")
            for name, cls in inspect.getmembers(mount_cls, inspect.isclass)
            if not name.startswith("_") and not name.endswith("Config")
        )
        super().__init__(
            config.get("command", mount_cls.__name__),
            main_args,
            config.get("prefixes", []),
            *_options,
            namespace=config.get("namespace", None),
            meta=CommandMeta(
                description=config.get("description", main_help_text),
                raise_exception=config.get("raise_exception", True),
            ),
            behaviors=[self.cb_behavior],
        )

    def get_result(self, func: Callable):
        return self.cb_behavior.results.get(func.__qualname__)


class ObjectMounter(Alconna[TDC], Generic[T, TDC]):
    mount_cls: Type[T]
    instance: T

    def __init__(self, obj: T, config: Optional[MountConfig] = None):
        self.mount_cls = type(obj)
        self.instance = obj
        config = config or visit_config(obj)
        obj_name = obj.__class__.__name__
        members = inspect.getmembers(
            obj, lambda x: inspect.isfunction(x) or inspect.ismethod(x)
        )
        _options = []
        main_help_text = obj.__doc__ or obj.__init__.__doc__ or obj_name

        def _main_func(**kwargs):
            for k, v in kwargs.items():
                setattr(self.instance, k, v)

        self.cb_behavior = CallbackHandler(main_call=_main_func)

        for name, func in filter(lambda x: not x[0].startswith("_"), members):
            help_text = func.__doc__ or name
            _opt_args, _ = Args.from_callable(func)
            _options.append(
                Option(
                    name, args=_opt_args, help_text=help_text
                )
            )
            self.cb_behavior.options[f"options.{name}.args"] = func

        _options.extend(
            SubClassMounter(cls, self.cb_behavior, "")
            for name, cls in inspect.getmembers(obj, inspect.isclass)
            if not name.startswith("_") and not name.endswith("Config")
        )
        main_args = Args.from_callable(obj.__init__)[0]
        for arg in main_args.argument:
            if hasattr(self.instance, arg.name):
                arg.field.default = getattr(self.instance, arg.name)

        super().__init__(
            config.get("command", obj_name),
            main_args,
            config.get("prefixes", []),
            *_options,
            meta=CommandMeta(
                description=config.get("description", main_help_text),
                raise_exception=config.get("raise_exception", True),
            ),
            behaviors=[self.cb_behavior],
            namespace=config.get("namespace", None),
        )

    def get_result(self, func: Callable):
        return self.cb_behavior.results.get(func.__qualname__)


@overload
def alconna_from_object(
    target: ModuleType, command: Optional[TDC] = None, config: Optional[MountConfig] = None,
) -> ModuleMounter:
    ...


@overload
def alconna_from_object(
    target: Type[T], command: Optional[TDC] = None, config: Optional[MountConfig] = None,
) -> ClassMounter[T, TDC]:
    ...


@overload
def alconna_from_object(
    target: Union[Callable[..., T], FunctionType], command: Optional[TDC] = None, config: Optional[MountConfig] = None,
) -> FuncMounter[T, TDC]:
    ...


@overload
def alconna_from_object(
    target: T, command: Optional[TDC] = None, config: Optional[MountConfig] = None,
) -> ObjectMounter[T, TDC]:
    ...


def alconna_from_object(  # type: ignore
    target: Optional[Union[Type[T], T, Callable[..., T], ModuleType]] = None,
    command: Optional[TDC] = None,
    config: Optional[MountConfig] = None,
) -> Union[
    ModuleMounter,
    ClassMounter[T, TDC],
    FuncMounter[T, TDC],
    ObjectMounter[T, TDC],
]:
    """
    通过解析传入的对象，生成 Alconna 实例的方法, 或者说是Fire-like的方式

    Examples:

        >>> def test_func(a, b, c):
        ...     print(a, b, c)
        ...
        >>> alc = AlconnaFire(test_func)
        >>> alc.parse("test_func 1 2 3")
    """
    if inspect.isfunction(target) or inspect.ismethod(target):
        r = FuncMounter(target, config)
    elif inspect.isclass(target):
        r = ClassMounter(target, config)
    elif inspect.ismodule(target):
        r = ModuleMounter(target, config)
    elif target:
        r = ObjectMounter(target, config)
    else:
        r = ModuleMounter(
            inspect.getmodule(inspect.stack()[1][0]) or sys.modules["__main__"], config
        )
    command = command or (" ".join(sys.argv[1:]) if len(sys.argv) > 1 else None)  # type: ignore
    if command:
        with suppress(Exception):
            r.parse(command)
        command_manager.require(r).reset()
    return r  # type: ignore


def delegate(cls: Type) -> Alconna:
    attrs = inspect.getmembers(cls, predicate=lambda x: not inspect.isroutine(x))
    _help = cls.__doc__ or cls.__name__
    _main_args = None
    _options = []
    _headers = []
    for name, attr in filter(lambda x: not x[0].startswith("_"), attrs):
        if isinstance(attr, Args):
            _main_args = attr
        elif isinstance(attr, (Option, Subcommand)):
            _options.append(attr)
        elif name.startswith("prefix"):
            _headers.extend(attr if isinstance(attr, (list, tuple)) else [attr])
    return Alconna(
        cls.__name__,
        _main_args,
        _headers,
        *_options,
        meta=CommandMeta(description=_help),
    )


def _argument(
    name: str,
    *alias: str,
    dest: Optional[str] = None,
    value: Optional[Any] = str,
    default: Optional[Any] = None,
    description: Optional[str] = None,
    required: bool = True,
    action: Optional[Action] = None,
):
    """类似于 argparse.ArgumentParser.add_argument() 的方法"""
    opt = Option(
        name, alias=list(alias), dest=dest, help_text=description, action=action
    )
    opt.args.add(
        name.strip("-"),
        value=value,  # type: ignore
        default=default,
        flags=[] if required else [ArgFlag.OPTIONAL],
    )
    opt.nargs += 1
    return opt


AlconnaFormat = alconna_from_format
AlconnaFire = alconna_from_object
Argument = _argument



from .config import lang as _lang  # noqa
from .construct import AlconnaDecorate as AlconnaDecorate
from .construct import AlconnaFire as AlconnaFire
from .construct import AlconnaFormat as AlconnaFormat
from .construct import AlconnaString as AlconnaString
from .construct import Argument as Argument
from .construct import Executor as Executor
from .construct import alconna_from_format
from .construct import alconna_from_object
from .construct import delegate as delegate
from .pattern import ObjectPattern as ObjectPattern
from .checker import simple_type as simple_type
from .actions import exclusion as exclusion
from .actions import cool_down as cool_down
from .actions import inclusion as inclusion
from .formatter import ShellTextFormatter as ShellTextFormatter
from .formatter import MarkdownTextFormatter as MarkdownTextFormatter
from .formatter import RichTextFormatter as RichTextFormatter
from .formatter import RichConsoleFormatter as RichConsoleFormatter



from __future__ import annotations

import traceback
from collections import namedtuple
from typing import Any, Literal

from arclet.alconna._internal._analyser import Analyser, default_compiler
from arclet.alconna._internal._handlers import analyse_args as ala
from arclet.alconna._internal._handlers import HEAD_HANDLES
from arclet.alconna._internal._handlers import analyse_option as alo
from arclet.alconna._internal._header import Header
from arclet.alconna.args import Args
from arclet.alconna.argv import Argv
from arclet.alconna.base import Option, Subcommand
from arclet.alconna.config import Namespace
from arclet.alconna.typing import DataCollection, CommandMeta


class AnalyseError(Exception):
    """分析时发生错误"""


dev_space = Namespace("devtool", enable_message_cache=False)


class _DummyAnalyser(Analyser):
    filter_out = []

    class _DummyALC:
        options = []
        meta = namedtuple("Meta", ["keep_crlf", "fuzzy_match", "raise_exception"])(False, False, True)
        namespace_config = dev_space

    def __new__(cls, *args, **kwargs):
        cls.command = cls._DummyALC()  # type: ignore
        cls.compile_params = {}
        cls.compact_params = []
        return super().__new__(cls)


def analyse_args(
    args: Args,
    command: list[str | Any],
    raise_exception: bool = True,
    context_style: Literal["bracket", "parentheses"] | None = None,
    **kwargs
):
    meta = CommandMeta(keep_crlf=False, fuzzy_match=False, raise_exception=raise_exception, context_style=context_style)
    argv = Argv(meta, dev_space)
    try:
        argv.enter(kwargs)
        argv.build(["test"] + command)
        argv.next()
        return ala(argv, args)
    except Exception as e:
        if raise_exception:
            traceback.print_exception(AnalyseError, e, e.__traceback__)
        return {}


def analyse_header(
    headers: list[str | Any] | list[tuple[Any, str]],
    command_name: str,
    command: DataCollection[str | Any],
    sep: str = " ",
    compact: bool = False,
    raise_exception: bool = True,
    context_style: Literal["bracket", "parentheses"] | None = None,
    **kwargs
):
    meta = CommandMeta(keep_crlf=False, fuzzy_match=False, raise_exception=raise_exception, context_style=context_style)
    argv = Argv(meta, dev_space, separators=(sep,))
    command_header = Header.generate(command_name, headers, compact=compact)
    try:
        argv.enter(kwargs)
        argv.build(command)
        return HEAD_HANDLES[command_header.flag](command_header, argv)
    except Exception as e:
        if raise_exception:
            traceback.print_exception(AnalyseError, e, e.__traceback__)
        return


def analyse_option(
    option: Option,
    command: DataCollection[str | Any],
    raise_exception: bool = True,
    context_style: Literal["bracket", "parentheses"] | None = None,
    **kwargs
):
    meta = CommandMeta(keep_crlf=False, fuzzy_match=False, raise_exception=raise_exception, context_style=context_style)
    argv = Argv(meta, dev_space)
    _analyser = _DummyAnalyser.__new__(_DummyAnalyser)
    _analyser.reset()
    _analyser.command.separators = (" ",)
    _analyser.need_main_args = False
    _analyser.command.options.append(option)
    default_compiler(_analyser, argv.param_ids)
    _analyser.command.options.clear()
    try:
        argv.enter(kwargs)
        argv.build(command)
        alo(_analyser, argv, option)
        return _analyser.options_result[option.dest]
    except Exception as e:
        if raise_exception:
            traceback.print_exception(AnalyseError, e, e.__traceback__)
        return


def analyse_subcommand(
    subcommand: Subcommand,
    command: DataCollection[str | Any],
    raise_exception: bool = True,
    context_style: Literal["bracket", "parentheses"] | None = None,
    **kwargs
):
    meta = CommandMeta(keep_crlf=False, fuzzy_match=False, raise_exception=raise_exception, context_style=context_style)
    argv = Argv(meta, dev_space)
    _analyser = _DummyAnalyser.__new__(_DummyAnalyser)
    _analyser.reset()
    _analyser.command.separators = (" ",)
    _analyser.need_main_args = False
    _analyser.command.options.append(subcommand)
    default_compiler(_analyser, argv.param_ids)
    _analyser.command.options.clear()
    try:
        argv.enter(kwargs)
        argv.build(command)
        return _analyser.compile_params[subcommand.name].process(argv).result()  # type: ignore
    except Exception as e:
        if raise_exception:
            traceback.print_exception(AnalyseError, e, e.__traceback__)
        return



"""Alconna ArgAction相关"""

from datetime import datetime
from tarina import lang
from typing import Literal
from dataclasses import dataclass, field
from arclet.alconna.exceptions import OutBoundsBehave
from arclet.alconna.arparma import Arparma, ArparmaBehavior


def exclusion(target_path: str, other_path: str):
    """
    当设置的两个路径同时存在时, 抛出异常

    Args:
        target_path: 目标路径
        other_path: 其他路径
    """

    class _EXCLUSION(ArparmaBehavior):
        def operate(self, interface: "Arparma"):
            if interface.query(target_path) and interface.query(other_path):
                raise OutBoundsBehave(
                    lang.require("tools", "actions.exclusion").format(left=target_path, right=other_path)
                )

    return _EXCLUSION()


def cool_down(seconds: float):
    """
    当设置的时间间隔内被调用时, 抛出异常

    Args:
        seconds: 时间间隔
    """

    @dataclass(unsafe_hash=True)
    class _CoolDown(ArparmaBehavior):
        last_time: datetime = field(default_factory=lambda: datetime.now())

        def operate(self, interface: "Arparma"):
            current_time = datetime.now()
            if (current_time - self.last_time).total_seconds() < seconds:
                raise OutBoundsBehave(lang.require("tools", "actions.cooldown"))
            else:
                self.last_time = current_time

    return _CoolDown()


def inclusion(*targets: str, flag: Literal["any", "all"] = "any"):
    """
    当设置的路径不存在时, 抛出异常

    Args:
        targets: 路径列表
        flag: 匹配方式, 可选值为"any"或"all", 默认为"any"
    """

    class _Inclusion(ArparmaBehavior):
        def operate(self, interface: "Arparma"):
            if flag == "all":
                for target in targets:
                    if not interface.query(target):
                        raise OutBoundsBehave(lang.require("tools", "actions.inclusion").format(target=target))
            else:
                all_count = len(targets) - sum(1 for target in targets if interface.query(target))
                if all_count > 0:
                    raise OutBoundsBehave(lang.require("tools", "actions.inclusion"))
    return _Inclusion()



from typing import List, Union, Tuple, Optional
from nepattern import Empty, ANY, AnyString
from tarina import lang
from arclet.alconna import AllParam
from arclet.alconna.args import Args, Arg
from arclet.alconna.base import Subcommand, Option, Shortcut, Completion
from arclet.alconna.formatter import TextFormatter, Trace, TraceHead


class ShellTextFormatter(TextFormatter):
    """
    shell 风格的帮助文本格式化器
    """

    def format(self, trace: Trace) -> str:
        parts = trace.body  # type: ignore
        sub_names = [i.name for i in parts if isinstance(i, Subcommand)]
        opt_names = [min(i.aliases, key=len) for i in parts if isinstance(i, Option) and i.name not in self.ignore_names]
        sub_names = f"{{{','.join(sub_names)}}}" if sub_names else ""
        opt_names = (" ".join(f"[{i}]" for i in opt_names) + "\n") if opt_names else ""
        topic = f"{lang.require('tools', 'format.ap.title')}: {trace.head['name']} {opt_names} {sub_names}"
        title, desc, usage, example = self.header(trace.head)
        param = self.parameters(trace.args)
        body = self.body(parts)
        res = topic
        if desc:
            res = f"{desc}\n\n{res}"
        if param:
            res += f"\n\n{lang.require('tools', 'format.ap.base')}: {title}{trace.separators[0]}{param}"
        if usage:
            res += f"\n{usage}"
        if body:
            res += f"\n\n{body}"
        if example:
            res += f"\n{example}"
        return res

    def param(self, parameter: Arg) -> str:
        name = parameter.name
        if str(parameter.value).strip("'\"") == name:
            return f"[{name}]" if parameter.optional else name
        if parameter.hidden:
            return f"[{name.upper()}]" if parameter.optional else name.upper()
        if parameter.value is AllParam:
            return f"{name.upper()}..."
        arg = f"[{name.upper()}" if parameter.optional else name.upper()
        if parameter.value not in (ANY, AnyString):
            arg += f"<{parameter.value}>"
        if parameter.field.display is not Empty:
            arg += f"={parameter.field.display}"
        return f"{arg}]" if parameter.optional else arg

    def parameters(self, args: Args) -> str:
        res = ""
        for arg in args.argument:
            if arg.name.startswith('_key_'):
                continue
            if len(arg.separators) == 1:
                sep = ' ' if arg.separators[0] == ' ' else f' {arg.separators[0]!r} '
            else:
                sep = f"[{'|'.join(arg.separators)!r}]"
            res += self.param(arg) + sep
        notice = [(arg.name, arg.notice) for arg in args.argument if arg.notice]
        return f"{res}\n{lang.require('tools', 'format.ap.notice')}:\n  - " + \
            "\n  - ".join(f"{v[0]}: {v[1]}" for v in notice) if notice else res

    def header(self, root: TraceHead):
        help_string = f"{desc}" if (desc := root.get("description")) else ""
        usage = f"{lang.require('tools', 'format.ap.usage')}: {usage}" if (usage := root.get("usage")) else ""
        example = f"{lang.require('tools', 'format.ap.example')}: {example}" if (example := root.get("example")) else ""
        return root["name"], help_string, usage, example

    def body(self, parts: List[Union[Option, Subcommand]]) -> str:
        options = []
        opt_description = []
        max_len = 1
        for opt in (i for i in parts if isinstance(i, Option) and not isinstance(i, (Completion, Shortcut))):
            name = (f'{{{" ".join(opt.requires)}}} ' if opt.requires else "") + ", ".join(sorted(opt.aliases, key=len))
            options.append(
                f"  {name}{tuple(opt.separators)[0]}{self.parameters(opt.args)}"
            )
            opt_description.append(opt.help_text)
        if options:
            max_len = max(max(map(lambda x: len(x), options)), max_len)
        subcommands = []
        sub_description = []
        for sub in (i for i in parts if isinstance(i, Subcommand)):
            name = (f'{{{" ".join(sub.requires)}}} ' if sub.requires else "") + ", ".join(sorted(sub.aliases, key=len))
            args = self.parameters(sub.args)
            subcommands.append(
                f"    {name}{tuple(sub.separators)[0]}{args}"
            )
            sub_topic = " ".join(f"[{i.name}]" for i in sub.options)  # type: ignore
            sub_description.append(f"{sub.help_text} {sub_topic}")
        if subcommands:
            max_len = max(max(map(lambda x: len(x), subcommands)), max_len)
        option_string = "\n".join(
            f"{i.ljust(max_len)}    {j}" for i, j in zip(options, opt_description)
        )
        subcommand_string = "\n".join(
            f"{i.ljust(max_len)}    {j}" for i, j in zip(subcommands, sub_description)
        )
        option_help = f"{lang.require('tools', 'format.ap.opt')}:\n{option_string}\n" if option_string else ""
        subcommand_help = f"{lang.require('tools', 'format.ap.sub')}:\n{subcommand_string}\n" if subcommand_string else ""
        return f"{subcommand_help}{option_help}"


class MarkdownTextFormatter(TextFormatter):
    def format(self, trace: Trace) -> str:
        """help text的生成入口"""
        """头部节点的描述"""
        root, separators = trace.head, trace.separators
        params, notice = self.parameters(trace.args)
        notice_text = (
            f"\n\n### {lang.require('format', 'notice')}:\n```\n" + "\n".join(notice) + "\n```"
        ) if notice else ""
        help_string = f"{desc}" if (desc := root.get("description")) else ""
        usage = f"{usage}" if (usage := root.get("usage")) else ""
        example = (
            f"## {lang.require('format', 'example')}:\n```shell\n{example}\n```"
            if (example := root.get("example"))
            else ""
        )
        command_string = root["name"].replace("[", "&#91;").replace("]", "&#93;")
        body = self.body(trace.body)
        res = (
            f"## {help_string}\n\n"
            f"### {lang.require('tools', 'format.md.title')}: \n\n"
            f"**{command_string}{separators[0]}{params}**{notice_text}"
        )
        if usage:
            res += f"\n\n{usage}"
        if body:
            res += f"\n\n{body}"
        if example:
            res += f"\n\n{example}"
        return res

    def param(self, parameter: Arg) -> str:
        """对单个参数的描述"""
        name = parameter.name
        if str(parameter.value).strip("'\"") == name:
            return f"&#91;{name}&#93;" if parameter.optional else name
        if parameter.hidden:
            return f"&#91;{name}&#93;" if parameter.optional else f"&lt;{name}&gt;"
        if parameter.value is AllParam:
            return f"&lt;...{name}&gt;"
        arg = f"&#91;{name}" if parameter.optional else f"&lt;{name}"
        if parameter.value not in (ANY, AnyString):
            arg += f": {parameter.value}"
        if parameter.field.display is not Empty:
            arg += f" = {parameter.field.display}"
        return f"{arg}&#93;" if parameter.optional else f"{arg}&gt;"

    def parameters(self, args: Args) -> Tuple[str, Optional[List[str]]]:
        """参数列表的描述"""
        res = ""
        for arg in args.argument:
            if arg.name.startswith('_key_'):
                continue
            if len(arg.separators) == 1:
                sep = ' ' if arg.separators[0] == ' ' else f' {arg.separators[0]!r} '
            else:
                sep = f"[{'|'.join(arg.separators)!r}]"
            res += self.param(arg) + sep
        notice = [(arg.name, arg.notice) for arg in args.argument if arg.notice]
        return (res[:-1], [f"{v[0]}: {v[1]}" for v in notice]) if notice else (res[:-1], None)

    def opt(self, node: Option) -> str:
        alias_text = " ".join(node.requires) + (" " if node.requires else "") + "│".join(node.aliases)
        help_text = "> Unknown" if node.help_text == node.dest else f"> {node.help_text}"
        param, notice = self.parameters(node.args)
        notice_text = (
            (f"\n>\n> #### {lang.require('format', 'notice')}:\n> " + "\n>\n> ".join(notice)) if notice else ""
        )
        return (
            f"- **{alias_text + (tuple(node.separators)[0] if param else '')}"
            f"{param.strip(' ')}**\n"
            f"{help_text}"
            f"{notice_text}\n"
        )

    def sub(self, node: Subcommand) -> str:
        """对单个子命令的描述"""
        name = " ".join(node.requires) + (" " if node.requires else "") + "│".join(node.aliases)
        opt_string = "\n".join(
            [self.opt(opt) for opt in node.options if isinstance(opt, Option)]
        )
        sub_string = "".join(
            [self.opt(sub) for sub in node.options if isinstance(sub, Subcommand)]  # type: ignore
        )
        opt_help = f"### {lang.require('format', 'subcommands.opts')}:\n" if opt_string else ""
        sub_help = f"### {lang.require('format', 'subcommands.subs')}:\n" if sub_string else ""
        param, notice = self.parameters(node.args)
        help_text = "> Unknown" if node.help_text == node.dest else f"> {node.help_text}"
        notice_text = (
            (f"\n>\n> #### {lang.require('format', 'notice')}:\n> " + "\n> ".join(notice)) if notice else ""
        )
        return (
            f"- **{name + (tuple(node.separators)[0] if param else '')}"
            f"{param}**\n"
            f"{help_text}"
            f"{notice_text}\n"
            f"{sub_help}{sub_string}"
            f"{opt_help}{opt_string}"
        )

    def body(self, parts: List[Union[Option, Subcommand]]) -> str:
        """子节点列表的描述"""
        option_string = "\n".join(
            [
                self.opt(opt) for opt in parts if isinstance(opt, Option) and opt.name not in self.ignore_names
            ]
        )
        subcommand_string = "\n".join(
            [self.sub(sub) for sub in parts if isinstance(sub, Subcommand)]
        )
        option_help = f"## {lang.require('format', 'options')}:\n" if option_string else ""
        subcommand_help = f"## {lang.require('format', 'subcommands')}:\n" if subcommand_string else ""
        return f"{subcommand_help}{subcommand_string}{option_help}{option_string}"


color_theme = {
    "msg": ("magenta", "35"),
    "warn": ("yellow", "33"),
    "info": ("blue", "34"),
    "error": ("red", "31"),
    "primary": ("cyan", "36"),
    "success": ("green", "32"),
    "req": ("bold green", "1;32")
}


class _RichTextFormatter(TextFormatter):
    csl_code: bool

    def _convert(self, content: str, style: str):
        if style not in color_theme:
            return content
        if self.csl_code:
            return f"\x1b[{color_theme[style][1]}m{content}\x1b[0m"
        content = content.replace("[", r"\[")
        return f"[{color_theme[style][0]}]{content}[/]"

    def format(self, trace: Trace) -> str:
        parts = trace.body  # type: ignore
        sub_names = [i.name for i in filter(lambda x: isinstance(x, Subcommand), parts)]
        opt_names = [min(i.aliases, key=len) for i in parts if isinstance(i, Option) and i.name not in self.ignore_names]
        sub_names = self._convert(f"{{{','.join(sub_names)}}}", "info") if sub_names else ""
        opt_names = self._convert((" ".join(f"[{i}]" for i in opt_names) + "\n"), 'info') if opt_names else ""
        title = f"{lang.require('tools', 'format.ap.title')}:"
        topic = f"{self._convert(title, 'warn')} {self._convert(trace.head['name'], 'msg')} {opt_names} {sub_names}"
        cmd, desc, usage, example = self.header(trace.head)
        param = self.parameters(trace.args)
        body = self.body(parts)
        res = topic
        if desc:
            res = f"{desc}\n\n{res}"
        if param:
            _base = lang.require('tools', 'format.ap.base')
            _param = self._convert(param, 'success')
            res += f"\n\n{self._convert(_base, 'warn')}: {cmd}{self._convert(trace.separators[0], 'msg')}{param}"
        if usage:
            res += f"\n{usage}"
        if body:
            res += f"\n\n{body}"
        if example:
            res += f"\n{example}"
        return res

    def param(self, parameter: Arg) -> str:
        name = parameter.name
        if str(parameter.value).strip("'\"") == name:
            return f"[{name}]" if parameter.optional else name
        if parameter.hidden:
            return f"[{name.upper()}]" if parameter.optional else name.upper()
        if parameter.value is AllParam:
            return f"{name.upper()}..."
        arg = f"[{name.upper()}" if parameter.optional else name.upper()
        if parameter.value not in (ANY, AnyString):
            arg += f"<{parameter.value}>"
        if parameter.field.display is not Empty:
            arg += f"={parameter.field.display}"
        return f"{arg}]" if parameter.optional else arg

    def parameters(self, args: Args) -> str:
        res = ""
        for arg in args.argument:
            if arg.name.startswith('_key_'):
                continue
            if len(arg.separators) == 1:
                sep = ' ' if arg.separators[0] == ' ' else f' {arg.separators[0]!r} '
            else:
                sep = f"[{'|'.join(arg.separators)!r}]"
            res += self.param(arg) + sep
        notice = [(arg.name, arg.notice) for arg in args.argument if arg.notice]
        _not = self._convert(f"{lang.require('tools', 'format.ap.notice')}:", 'warn')
        return f"{res}\n{_not}\n  - " + \
            "\n  - ".join(self._convert(f"{v[0]}: {v[1]}", "success") for v in notice) if notice else res

    def header(self, root: TraceHead):
        _usage = f"{lang.require('tools', 'format.ap.usage')}:"
        _example = f"{lang.require('tools', 'format.ap.example')}:"
        help_string = f"{desc}" if (desc := root.get("description")) else ""
        usage = f"{self._convert(_usage, 'warn')} {usage}\n" if (usage := root.get("usage")) else ""
        example = f"{self._convert(_example, 'warn')} {example}\n" if (example := root.get("example")) else ""
        command_string = self._convert(root["name"], "msg")
        return command_string, help_string, usage, example

    def body(self, parts: List[Union[Option, Subcommand]]) -> str:
        options = []
        opt_description = []
        max_len = 1
        for opt in (i for i in parts if isinstance(i, Option) and not isinstance(i, (Completion, Shortcut))):
            name = (f'{{{" ".join(opt.requires)}}} ' if opt.requires else "") + ", ".join(sorted(opt.aliases, key=len))
            options.append(
                self._convert(
                    f"  {name}{tuple(opt.separators)[0]}{self.parameters(opt.args)}",
                    "primary"
                )
            )
            opt_description.append(opt.help_text)
        if options:
            max_len = max(max(map(lambda x: len(x), options)), max_len)
        subcommands = []
        sub_description = []
        for sub in (i for i in parts if isinstance(i, Subcommand)):
            name = (f'{{{" ".join(sub.requires)}}} ' if sub.requires else "") + ", ".join(sorted(sub.aliases, key=len))
            args = self.parameters(sub.args)
            subcommands.append(
                self._convert(
                    f"    {name}{tuple(sub.separators)[0]}{args}",
                    "primary"
                )
            )
            sub_topic = " ".join(f"[{i.name}]" for i in sub.options)  # type: ignore
            sub_description.append(f"{sub.help_text} {sub_topic}")
        if subcommands:
            max_len = max(max(map(lambda x: len(x), subcommands)), max_len)
        option_string = "\n".join(
            f"{i.ljust(max_len)}    {j}" for i, j in zip(options, opt_description)
        )
        subcommand_string = "\n".join(
            f"{i.ljust(max_len)}    {j}" for i, j in zip(subcommands, sub_description)
        )
        _opt = f"{lang.require('tools', 'format.ap.opt')}:"
        _sub = f"{lang.require('tools', 'format.ap.sub')}:"
        option_help = f"{self._convert(_opt, 'warn')}\n{option_string}\n" if option_string else ""
        subcommand_help = f"{self._convert(_sub, 'warn')}\n{subcommand_string}\n" if subcommand_string else ""
        return f"{subcommand_help}{option_help}"


class RichTextFormatter(_RichTextFormatter):
    """argparser 风格的帮助文本格式化器, 增加 rich 的颜色标记，可用 rich.console 打印"""
    csl_code = False


class RichConsoleFormatter(_RichTextFormatter):
    """argparser 风格的帮助文本格式化器, 增加控制台颜色标记"""
    csl_code = True
